Level01 = Core.class(Sprite)

function Level01:init()
	-- the plugin
	require "liquidfun"
	--b2.setScale(32) -- gideros default: 30px = 1m

	-- the bg
	application:setBackgroundColor(0x0aafaf)

	-- the liquidfun world
	self.world = b2.World.new(0, 9.8, true)

	-- the ground
	local groundtexture = Texture.new("gfx/world/mytile32.png", true, {wrap = Texture.REPEAT})
	local levelmatrix = Matrix.new(1, 0, 0, 1, 0, 0)
	local levelcoords = {
		{x=-31,y=-3},
		{x=-31,y=4},
		{x=-20,y=4},
		{x=-14,y=0},
		{x=-9,y=0},
		{x=-9,y=-2},
		{x=-6,y=-2},
		{x=-4,y=0},
		{x=1,y=0},
		{x=6,y=4},
		{x=21,y=4},
		{x=21,y=-3},
		{x=23,y=-3},
		{x=23,y=4},
		{x=31,y=4},
		{x=31,y=-3},
		{x=33,y=-3},
		{x=33,y=8},
		{x=-33,y=5},
		{x=-33,y=-3},
		{x=-31,y=-3},
	}

	local levelscale = 32 -- the scale of the level coors

	local levelshape = Shape.new()
	levelshape:setFillStyle(Shape.TEXTURE, groundtexture, levelmatrix)
	levelshape:beginPath()
	levelshape:moveTo(levelcoords[1].x * levelscale, levelcoords[1].y * levelscale)
	for p = 2, #levelcoords do
		levelshape:lineTo(levelcoords[p].x * levelscale, levelcoords[p].y * levelscale)
	end
	levelshape:closePath()
	levelshape:endPath()
	levelshape:setPosition(levelshape:getWidth() / 2, levelshape:getHeight() / 2)

	self.levelbody = self.world:createBody {type = b2.STATIC_BODY}
	self.levelbody:setPosition(levelshape:getX(), levelshape:getY())
	local levelchain = b2.ChainShape.new()
	levelchain:createLoop(
		-31*levelscale,-3*levelscale,
		-31*levelscale,4*levelscale,
		-20*levelscale,4*levelscale,
		-14*levelscale,0*levelscale,
		-9*levelscale,0*levelscale,
		-9*levelscale,-2*levelscale,
		-6*levelscale,-2*levelscale,
		-4*levelscale,0*levelscale,
		1*levelscale,0*levelscale,
		6*levelscale,4*levelscale,
		21*levelscale,4*levelscale,
		21*levelscale,-3*levelscale,
		23*levelscale,-3*levelscale,
		23*levelscale,4*levelscale,
		31*levelscale,4*levelscale,
		31*levelscale,-3*levelscale,
		33*levelscale,-3*levelscale,
		33*levelscale,8*levelscale,
		-33*levelscale,5*levelscale,
		-33*levelscale,-3*levelscale,
		-31*levelscale,-3*levelscale
	)
	local levelfixture = self.levelbody:createFixture { shape = levelchain, density = 1, friction = 0.1, restitution = 0 }

	-- a platform
	self.myplatform2 = LF_Platform.new(self.world, "gfx/platforms/mytile32.png", 6, 1, 0, 2)
	self.myplatform2.startx, self.myplatform2.starty = 1759, 100
	self.myplatform2:setPosition(self.myplatform2.startx, self.myplatform2.starty)
	self:addChild(self.myplatform2)
	self:addChild(levelshape)

	-- the player
	self.player = Player.new(self.world, "gfx/player/HQ_Trooper_all.png", 6, 13)
	self.player.lives = 3
	self.player.body:setPosition(1600, contentheight / 2) -- position the body, the bitmap will follow in enterframe
	self:addChild(self.player)

	-- enemies
	self.enemy = LF_Body.new(self.world, b2.DYNAMIC_BODY, "gfx/npcs/ghost.png", 1, 1, true, 0.5, 0.7)
	self.enemy.body:setPosition(450, 0) -- position the body, the bitmap will follow in enterframe
	self:addChild(self.enemy)
	self.enemy02 = LF_Body.new(self.world, b2.DYNAMIC_BODY, "gfx/npcs/ghost.png", 0.5, 0.5, true, 1, 0.1)
	self.enemy02.body:setPosition(1000, 0) -- position the body, the bitmap will follow in enterframe
	self:addChild(self.enemy02)

	-- liquidfun debug
	local debugdraw = b2.DebugDraw.new()
	self.world:setDebugDraw(debugdraw)
--	self:addChild(debugdraw)

	-- LISTENERS
	self.world:addEventListener(Event.BEGIN_CONTACT, self.onBeginContact, self)
	self.world:addEventListener(Event.END_CONTACT, self.onEndContact, self)
	self.world:addEventListener(Event.PRE_SOLVE, self.onPreSolveContact, self)
	self.world:addEventListener(Event.POST_SOLVE, self.onPostSolveContact, self)
	self:addEventListener("enterBegin", self.onTransitionInBegin, self)
	self:addEventListener("enterEnd", self.onTransitionInEnd, self)
	self:addEventListener("exitBegin", self.onTransitionOutBegin, self)
	self:addEventListener("exitEnd", self.onTransitionOutEnd, self)
end

--define collision handler
function Level01:onBeginContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	if fixtureA.type == 2 or fixtureB.type == 2 then
		-- is on floor
		self.player.numfeetcontacts += 1
	end
end

function Level01:onEndContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	if fixtureA.type == 2 or fixtureB.type == 2 then
		-- is no more on floor
		self.player.numfeetcontacts -= 1
	end
end

function Level01:onPreSolveContact(e)
	local fixtureA = e.fixtureA
	local fixtureB = e.fixtureB
	local platformfixture =  nil
	local otherfixture = nil
	if fixtureA == self.myplatform2.fixture then
		platformfixture = fixtureA
		otherfixture = fixtureB
	elseif fixtureA == self.myplatform2.fixture then
		platformfixture = fixtureB
		otherfixture = fixtureA
	end
	if not platformfixture then return end

	-- pass through platform
	if otherfixture:getBody() == self.player.body then
		local velocityx, velocityy = otherfixture:getBody():getLinearVelocity()
		if velocityy > 0 then -- going down = collision
			e.contact:setEnabled(true)
			return
		end
		e.contact:setEnabled(false) -- else going up = no collision
	end
end

function Level01:onPostSolveContact(e)
end

-- the game loop
function Level01:onEnterFrame(e)
	self:updatePlatforms(e.deltaTime)
	self:updatePlayer(e.deltaTime)
	self:updateEnemy(e.deltaTime)
	self.world:step(1/60, 1, 1)
end

-- update player
function Level01:updatePlayer(dt)
	local playerposx, playerposy = self.player.body:getPosition()
	local playervelx, playervely = self.player.body:getLinearVelocity()

	-- move & flip player sprite along with player physic body
	self.player:setPosition(playerposx, playerposy)
	self.player:setRotation(self.player.body:getAngle() * 180 / math.pi)
	self.player.bitmap:setScale(self.player.flip, 1)

	-- camera follow player
	local offsetX = -(playerposx - contentwidth / 2)
	self:setX(offsetX)
	local offsetY = -(playerposy - contentheight / 2)
	self:setY(offsetY)

	-- anim state
	if self.player.numfeetcontacts > 0 then -- is on ground
		if playervelx == 0 then
			self.player.currentanim = "idle"
		elseif playervelx ~= 0 then
			self.player.currentanim = "walk"
		end
	else
		if playervely < 0 then
			self.player.currentanim = "jump_up"
		elseif playervely >= 0 then
			self.player.currentanim = "jump_down"
		end
	end

--[[
	-- 1st method player controls velocity direct and gradual -- PROBLEM STICK ON WALLS
	if self.player.isleft and not self.player.isright then
--		playervelx = -self.player.speed
		playervelx = math.max(playervelx - 0.15, -self.player.speed)
	elseif self.player.isright and not self.player.isleft then
--		playervelx = self.player.speed
		playervelx = math.min(playervelx + 0.15, self.player.speed)
	end
	self.player.body:setLinearVelocity(playervelx, playervely)
]]

--[[
	-- 2nd method player controls force direct and gradual -- NO PROBLEM STICK ON WALLS
	local forcex, forcey = 0, 0
	if self.player.isleft and not self.player.isright then
--		playervelx = -self.player.speed
		if playervelx > -self.player.speed then forcex = -self.player.speed * 5 end
		self.player.flip = -1
	elseif self.player.isright and not self.player.isleft then
--		playervelx = self.player.speed
		if playervelx < self.player.speed then forcex = self.player.speed * 5 end
		self.player.flip = 1
	end
--	self.player.body:applyForce(forcex, 0, self.player.body:getWorldCenter())
]]
--[[
	-- jump with force ***
	if self.player.remainingjumpsteps > 0 then
		-- 1
		forcey = -20 * self.player.speed
		self.player.body:applyForce(0, forcey, self.player.body:getWorldCenter())
		self.player.remainingjumpsteps -= 1
		-- 2
		forcey = -self.player.body:getMass() * 1.5 * self.player.speed / (1 / 60)
		forcey /= 6
		self.player.body:applyForce(0, forcey, self.player.body:getWorldCenter())
		self.player.remainingjumpsteps -= 1
	end
]]

	-- 3rd method player controls impulse direct -- NO PROBLEM STICK ON WALLS BRENT AURELI SUPER MARIO
--	local impulse = self.player.body:getMass() * 7.5
	local impulse = 1
	if self.player.isleft and not self.player.isright and playervelx > -self.player.speed then
		self.player.body:applyLinearImpulse(-impulse, 0, self.player.body:getWorldCenter())
		self.player.flip = -1
	end
	if self.player.isright and not self.player.isleft and playervelx < self.player.speed then
		self.player.body:applyLinearImpulse(impulse, 0, self.player.body:getWorldCenter())
		self.player.flip = 1
	end
	
	-- anim loop
	if self.player.currentanim ~= "" then
		self.player.animtimer = self.player.animtimer - dt
		if self.player.animtimer <= 0 then
			self.player.frame += 1
			self.player.animtimer = self.player.animspeed
			if self.player.frame > #self.player.anims[self.player.currentanim] then
				self.player.frame = 1
			end
			self.player.bitmap:setTextureRegion(self.player.anims[self.player.currentanim][self.player.frame])
		end
	end
end

-- update enemy
function Level01:updateEnemy(dt)
	self.enemy.bitmap:setPosition(self.enemy.body:getPosition())
	self.enemy.bitmap:setRotation(self.enemy.body:getAngle() * 180 / math.pi)
	self.enemy02.bitmap:setPosition(self.enemy02.body:getPosition())
	self.enemy02.bitmap:setRotation(self.enemy02.body:getAngle() * 180 / math.pi)
end

-- update platforms
function Level01:updatePlatforms(dt)
	local myplatform2posx, myplatform2posy = self.myplatform2.body:getPosition()
	if myplatform2posy <= self.myplatform2.starty then
		self.myplatform2.body:setLinearVelocity(self.myplatform2.lvx, self.myplatform2.lvy)
	elseif myplatform2posy >= self.myplatform2.starty + 190 then
		self.myplatform2.body:setLinearVelocity(self.myplatform2.lvx, -self.myplatform2.lvy)
	end
	self.myplatform2.mybitmap:setPosition(myplatform2posx, myplatform2posy)
end

-- EVENT LISTENERS
function Level01:onTransitionInBegin()
	self:myKeysPressed()
end

function Level01:onTransitionInEnd()
	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function Level01:onTransitionOutBegin()
	self:removeEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function Level01:onTransitionOutEnd()
end

-- KEYS HANDLER
function Level01:myKeysPressed()
	self:addEventListener(Event.KEY_DOWN, function(e)
		-- for mobiles and desktops
		if e.keyCode == KeyCode.BACK or e.keyCode == KeyCode.ESC then
--			sceneManager:changeScene("menu", 1, transitions[2], easing.outBack)
		end
	end)
end
